package xyz.leomurca.csgomatches.ui.components

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.drawBehind
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import coil3.compose.AsyncImage
import xyz.leomurca.csgomatches.R
import xyz.leomurca.csgomatches.domain.model.League
import xyz.leomurca.csgomatches.domain.model.Match
import xyz.leomurca.csgomatches.domain.model.MatchStatus
import xyz.leomurca.csgomatches.domain.model.Opponent
import xyz.leomurca.csgomatches.ui.theme.LiveRed
import xyz.leomurca.csgomatches.ui.theme.White_20
import xyz.leomurca.csgomatches.ui.theme.White_50
import xyz.leomurca.csgomatches.utils.toFormattedMatchTime
import java.time.ZonedDateTime

@Composable
fun MatchCard(
    match: Match,
    modifier: Modifier = Modifier,
) {
    Box(modifier = modifier.fillMaxWidth()) {
        Card(
            modifier = Modifier.fillMaxWidth(),
            shape = RoundedCornerShape(16.dp),
            colors = CardDefaults.cardColors(containerColor = MaterialTheme.colorScheme.surface)
        ) {
            MatchupRow(match.opponents)
            LeagueInfoRow(match.league, match.serie.name)
        }
        ScheduleBadge(match.status, match.beginAt)
    }
}

@Composable
private fun MatchupRow(opponents: List<Opponent>) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .padding(top = 43.5.dp, bottom = 18.5.dp),
        horizontalArrangement = Arrangement.Center,
        verticalAlignment = Alignment.CenterVertically
    ) {
        val (leftOpponent, rightOpponent) = getOrDefaultOpponents(opponents)

        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            AsyncImage(
                model = leftOpponent.imageUrl,
                contentDescription = "${leftOpponent.name} logo",
                error = painterResource(R.drawable.fallback_image_round),
                placeholder = painterResource(R.drawable.fallback_image_round),
                modifier = Modifier.size(60.dp),
                contentScale = ContentScale.Fit
            )
            Text(
                leftOpponent.name,
                style = MaterialTheme.typography.labelMedium,
                modifier = Modifier.padding(top = 10.dp)
            )
        }

        Text(
            "vs",
            style = MaterialTheme.typography.bodyMedium,
            color = White_50,
            modifier = Modifier.padding(horizontal = 20.dp)
        )

        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            AsyncImage(
                model = rightOpponent.imageUrl,
                contentDescription = "${rightOpponent.name} logo",
                error = painterResource(R.drawable.fallback_image_round),
                placeholder = painterResource(R.drawable.fallback_image_round),
                modifier = Modifier.size(60.dp),
                contentScale = ContentScale.Fit
            )
            Text(
                rightOpponent.name,
                style = MaterialTheme.typography.labelMedium,
                modifier = Modifier.padding(top = 10.dp)
            )
        }
    }
}

@Composable
private fun LeagueInfoRow(league: League, serieName: String) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = Modifier
            .fillMaxWidth()
            .topBorder(White_20, 1.dp)
            .padding(vertical = 8.dp, horizontal = 16.dp)
    ) {
        AsyncImage(
            model = league.imageUrl,
            contentDescription = "${league.name} logo",
            error = painterResource(R.drawable.fallback_image_round),
            placeholder = painterResource(R.drawable.fallback_image_round),
            onLoading = {
            },
            modifier = Modifier
                .height(16.dp)
                .wrapContentWidth(),
            contentScale = ContentScale.Fit
        )
        Text(
            "${league.name} + $serieName",
            style = MaterialTheme.typography.labelSmall,
            modifier = Modifier.padding(start = 8.dp)
        )
    }
}

@Composable
private fun BoxScope.ScheduleBadge(status: MatchStatus, beginAt: ZonedDateTime?) {
    when (status) {
        MatchStatus.LIVE ->
            Box(
                modifier = Modifier
                    .align(Alignment.TopEnd)
                    .clip(RoundedCornerShape(topEnd = 16.dp, bottomStart = 16.dp))
                    .background(LiveRed)
                    .padding(8.dp)
            ) {
                Text(
                    text = "AGORA",
                    style = MaterialTheme.typography.displayMedium,
                    color = Color.White
                )
            }

        MatchStatus.SCHEDULED ->
            Box(
                modifier = Modifier
                    .align(Alignment.TopEnd)
                    .clip(RoundedCornerShape(topEnd = 16.dp, bottomStart = 16.dp))
                    .background(White_20)
                    .padding(8.dp)
            ) {
                Text(
                    text = beginAt.toFormattedMatchTime(),
                    style = MaterialTheme.typography.displayMedium,
                    color = Color.White
                )
            }

        MatchStatus.UNKNOWN ->
            Box(
                modifier = Modifier
                    .align(Alignment.TopEnd)
                    .clip(RoundedCornerShape(topEnd = 16.dp, bottomStart = 16.dp))
                    .background(White_20)
                    .padding(8.dp)
            ) {
                Text(
                    text = "A ser definido",
                    style = MaterialTheme.typography.displayMedium,
                    color = Color.White
                )
            }
    }
}

private fun Modifier.topBorder(color: Color, thickness: Dp): Modifier = this.then(
    Modifier.drawBehind {
        val strokeWidth = thickness.toPx()
        drawLine(
            color = color,
            start = Offset(0f, 0f),
            end = Offset(size.width, 0f),
            strokeWidth = strokeWidth
        )
    }
)

private fun getOrDefaultOpponents(opponents: List<Opponent>): Pair<Opponent, Opponent> {
    val default = Opponent(id = 0, name = "A ser definido", imageUrl = "")
    return when {
        opponents.size >= 2 -> opponents[0] to opponents[1]
        opponents.size == 1 -> opponents[0] to default
        else -> default to default
    }
}

